@@ -1,8 +1,19 @@
+{
+  package AccessorGroups::BeenThereDoneThat;
+
+  use strict;
+  use warnings;
+  use base 'Class::Accessor::Grouped';
+
+  __PACKAGE__->mk_group_accessors('simple', 'singlefield');
+  __PACKAGE__->mk_group_accessors('multiple', qw/multiple1 multiple2/);
+}
+
+
 package AccessorGroups;
 use strict;
 use warnings;
 use base 'Class::Accessor::Grouped';
-
 __PACKAGE__->mk_group_accessors('simple', 'singlefield');
 __PACKAGE__->mk_group_accessors('multiple', qw/multiple1 multiple2/);
 __PACKAGE__->mk_group_accessors('listref', [qw/lr1name lr1;field/], [qw/lr2name lr2'field/]);
